clear
//stacksize('max'); // for scilab 5 only
warning('off')

exec("config_display.sce");
exec("optimization_display.sci");

// to manage pauses
function go_on_box()
    btn = messagebox("Continue ?", "Scilab Message", "question", ["Continue" "Stop"], "modal");
    if btn==2 then 
        abort
    end
endfunction

// init 
figs = list(); // list of figure handles

//------------------------------------
// NIAC sphere / circle 
if map.type=="full" then
    SImapFile = results_dir + "fullNIACmap" + string(NbChannels) + "srcs.dat";
else
    SImapFile = results_dir + "simplifiedNIACmap" + string(NbChannels) + "srcs.dat";
end
select NbChannels
case 2
    if map.type=="full" then
        [figs($+1),map.minJ,map.maxJ] = SIcircle(SImapFile, sources2, map);
    else
        figs($+1) = simplified_SIcircle(SImapFile,sources2,map); 
    end    
case 3
    if map.type=="full" then
        figs($+1) = ConstraintsSphere(SImapFile); 
        figs($+1) = SIsphere(SImapFile,sources3); 
    else
        figs($+1) = simplified_SIsphere(SImapFile,sources3); 
    end
else
    simplified_SIdisplay(SImapFile);    
end

go_on_box();

//----------------------------------
// Display 1st optimization process, for 2 or 3 sources
opt1fileParams = results_dir+direction+"1NIAC_"+string(NbChannels)+"srcs_dataParams.dat";
load(opt1fileParams);
opt1file = results_dir+direction+"1NIAC_"+string(NbChannels)+"srcs.dat";
load(opt1file);
if NbChannels <= 3 then
    figs($).figure_name = "1st optimization : " + string(paramOpt1.type);
    select paramOpt1.type
    case "PSO"
        last_points = display_1stOpt_PSO(points,NbChannels,figs($),2,map);
    case "GradDesc"
        last_points = display_1stOpt_GD(points,NbChannels,figs($),2,map); 
    case "BFGS"
        points_norm = points;
        for i=1:size(points,2)
            points_norm(1:NbChannels,i) = points(1:NbChannels,i) / norm(points(1:NbChannels,i));
        end
        last_points = display_1stOpt_GD(points,NbChannels,figs($),2,map); 
    end    
    go_on_box();
end

//-------------------------------
// 2nd optimization

// load max2NIAC
BetaMax1 = BetaMax; // Opt from the 1st optimization = start point for 2nd optimization
points1 = points;
opt2file = results_dir+direction+"2NIAC_"+string(NbChannels)+"srcs.dat";
load(opt2file);
// normalize points to keep them on the sphere/circle
points_norm = points;
for i=1:size(points,2)
    points_norm(1:NbChannels,i) = points(1:NbChannels,i) / norm(points(1:NbChannels,i));
end
    
if NbChannels <= 3 then

    select NbChannels
        
    case 2
        R = eye(2,2);
        // zoom and display optimization points on current fig
        if map.type=="full" then
            SImapFile = results_dir + "fullNIACmap" + string(NbChannels) + "srcs_zoom_"+direction+".dat";
            // display zooming angular sector
            disp_zoomArea(SImapFile,figs($),map);
            go_on_box();
            // on this sector, disp NIAC as a function of arctan(beta2/beta1)
            figs($+1) = zoomed_SIcurve(SImapFile,BetaRef);
            // display 2nd optimization
            display_2ndOpt(points_norm,BFGSpoints,R,1,map);
        else
            // display zooming rectangle
            zoomRect = disp_zoomArea_simple(points_norm,BetaRef,figs($),map);
            go_on_box();
            // zoom
            zoomed_SImap_simple(figs($),zoomRect,last_points);
            // display 2nd optimization
            display_2ndOpt(points_norm,BFGSpoints,R,1,map);
        end

    case 3
        if map.type=="full" then
             // rotate the NIAC sphere (current fig) so that BetaMax1 is on the center of the view
             // and indicate the zoom area for the second optimization
             SImapFile = results_dir + "fullNIACmap" + string(NbChannels) + "srcs_zoom_"+direction+".dat";
             SImap(points_norm,BFGSpoints,BetaMax1,BetaRef,figs($));
             go_on_box();
             // display 2nd optimization points on zoomed SImap 
             figs($+1) = zoomed_SImap(SImapFile,BetaRef,map);
             load(SImapFile,'R');    // to rotate points of the optimization process
             display_2ndOpt(points_norm,BFGSpoints,R,1,map);
        else
            // rotate sphere and project it such that BetaMax1 is on the center
             // and indicate the zoom area for the second optimization
            [figs($+1),last_points,R,zoomRect] = SImap_simple(SImapFile,points1,BetaMax1,points_norm,BetaRef,map,sources3);
            go_on_box();
            // zoom and display optimization points
            zoomed_SImap_simple(figs($),zoomRect,last_points);
            display_2ndOpt(points_norm,BFGSpoints,R,1,map);
    
        end
    end
   
end


disp(results);







