function [R,U] = matRotation(a,b)
    // Considering 2 unit column vectors a and b 
    // Find the rotation matrix of center O transforming a into b
    // We express it as a rotation of center O in the plane OAB
    // -> base change so that the 2 1st vectors define the plane OAB
    p = length(a); 
    
    // base change
    V = [a,[eye(p-1,p-1);zeros(1,p-1)]]; // pre-condition : a not in {e2,...e_p}
    U = zeros(p,p); // new base
    U(:,1) = a;
    v = b - (a'*b)*a;   // such that v orth to u and v in plan OAB
    U(:,2) = v / norm(v); 
    // For the next vectors of base U,
    // Gram-Schmidt orthonormalization
    for i=3:p
        U(:,i) = V(:,i);
        for j=1:p-1
            U(:,i) = U(:,i) - (U(:,j)'*V(:,i))*U(:,j);
        end
        U(:,i) = U(:,i) / norm(U(:,i));
    end
    
    // Rotation matrix in the new base U
    theta = acos(a'*b);
    Rplan = [cos(theta),-sin(theta);sin(theta),cos(theta)];
    Rbis = [[Rplan,zeros(2,p-2)];[zeros(p-2,2),eye(p-2,p-2)]];
    
    // Rotation matrix in the canonical base
    R = U*Rbis*U';
    
endfunction
