//*****************
// Optimization in dim p for p sources :
//
// NIAC spheres and circles :
//- SIsphere and ConstraintsSphere-> build_facets and facet
//- SIcircle
//- simplified_SIsphere
//- simplified_SIcircle
//
// NIAC maps : projection + display zoom area
//- SImap 
//- SImap_simple 
//- disp_zoomArea
//- disp_zoomArea_simple
//
// zoomed maps :
//- zoomed_SImap
//- zoomed_SImap_simple
//- zoomed_SIcurve
//
//Optimization swarms and trajectories :
//- display_1stOpt_GD    
//- display_1stOpt_PSO
//- display_2ndOpt //  for 2nd optimization (BFGS)
//
//***********************



//************** NIAC spheres and circles *****************

function fig = ConstraintsSphere(SImapFile)
    // display constraints on the sphere of radius 1
    [X,Y,Z,JJ,CC] = build_facets(SImapFile);
    fig=scf();
    fig.color_map = graycolormap(3);
    plot3d(X,Y,list(Z,CC+1),flag=[1,6,3]);
    a = gca();
    xlabel("$\beta_1$"); a.x_label.font_size = 4;
    ylabel("$\beta_2$"); a.y_label.font_size = 4;
    zlabel("$\beta_3$"); a.z_label.font_size = 4;
    fig.figure_name = "Sign constraints"
    xtitle("Forbidden area in black")
endfunction


function fig = SIsphere(SImapFile,sources)
    // display the NIAC values as colors on the sphere of radius 1
    // INPUTS :
    // - SImapFile = data file
    // - sources = list of sources (strings)
    [X,Y,Z,JJ,CC] = build_facets(SImapFile);
    fig=scf();
    fig.color_map = jetcolormap(256);
    // sphere colored with cold colors (keep cold ones for points display)
    colours = round(127*(JJ-min(JJ))/(max(JJ)-min(JJ)));
    plot3d(X,Y,list(Z,colours),flag=[1,6,3])
    // extraction points, with indication whether they are min / max / uncertain
    load(SImapFile); // -> 'extraction_points','estMax','pSI','Phi','Thetas','Betas','Jmean','constraints'
    p = list();
    for k=1:3
        x = extraction_points(1,k);
        y = extraction_points(2,k);
        z = extraction_points(3,k);
        plot3d([x -x]*1.02,[y -y]*1.02,[z -z]*1.02,flag=[0,0]);
        p($+1) = gce();
        p($).surface_mode = "off";
        p($).mark_mode = "on";
        select estMax(k)
        case 1
            p($).mark_style = 6;
        case -1
            p($).mark_style = 7;
        case 0
           p($).mark_style = 9;
        end
        p($).thickness = 2;
        p($).mark_size = 2;
        p($).mark_foreground = 165+30*k;
        p($).mark_background = 165+30*k;                   
    end   
    // legend
    legends([sources(1),sources(2),sources(3)], [-p(1).mark_style, -p(2).mark_style, -p(3).mark_style], "ur") 
    e = gce();
    for k=1:3
        e.children(2*k).mark_foreground = 0;
        e.children(2*k).mark_background = p(k).mark_background;
        e.children(2*k).mark_size = 6;
        e.children(2*k-1).font_size = 3
    end
    // layout
    a = gca();
    a.margins =  [0.1,0.1,0.1,0.1];
    xtitle("NIAC map");
    a.title.font_size = 4;
    xlabel("$\beta_1$"); a.x_label.font_size = 4;
    ylabel("$\beta_2$"); a.y_label.font_size = 4;
    zlabel("$\beta_3$"); a.z_label.font_size = 4;
endfunction

function [X,Y,Z,JJ,CC] = build_facets(SImapFile)
    // build facets of the NIAC sphere
    load(SImapFile); // -> 'extraction_points','estMax','pSI','Phi','Thetas','Betas','Jmean','constraints'
    X = []; Y = []; Z = []; JJ = []; CC = [];
    // build facets around south pole
    Pinf = Betas(1);
    Psup = [Betas(2) Betas(2)(:,1)];
    Jinf = Jmean(1);
    Jsup = [Jmean(2) Jmean(2)(1)];
    Cinf = constraints(1);
    Csup = [constraints(2) constraints(2)(1)];
    for j=1:size(Betas(2),2)
        [x,y,z,J,C] = facet(Pinf,Psup(:,j),Psup(:,j+1),Jinf,Jsup(j),Jsup(j+1),Cinf,Csup(j),Csup(j+1));
        X = [X x]; Y = [Y y]; Z = [Z z]; JJ = [JJ J]; CC = [CC C];
    end
    // build facets btw poles
    for i_phi = 2:length(Phi)-2
        Pinf = [Betas(i_phi) Betas(i_phi)(:,1)]; // matrix of points on parallel inf
        Psup = [Betas(i_phi+1) Betas(i_phi+1)(:,1)]; // matrix of points on parallel sup
        Jinf = [Jmean(i_phi) Jmean(i_phi)(1)]; 
        Jsup = [Jmean(i_phi+1) Jmean(i_phi+1)(1)]; 
        Cinf = [constraints(i_phi) constraints(i_phi)(1)]; 
        Csup = [constraints(i_phi+1) constraints(i_phi+1)(1)]; 
        while size(Pinf,2)+size(Psup,2) >=3 // while possible to draw a triangle
            if size(Pinf,2)>1 then
                dInf = norm(Pinf(:,2)-Pinf(:,1)) + norm(Pinf(:,2)-Psup(:,1));
            else 
                dInf = %inf;
            end
            if size(Psup,2)>1 then
                dSup = norm(Psup(:,2)-Pinf(:,1)) + norm(Psup(:,2)-Psup(:,1));
            else 
                dSup = %inf;
            end
            if dInf < dSup
                //[x,y,z,J,C] = facet(Pinf(:,1),Pinf(:,2),Psup(:,1),Jinf(1),Jinf(2),Jsup(1),Cinf(1),Cinf(2),Csup(1)); // non, il faut que ce soit dans le sens anti-horaire, sinon c'est la face intérieure de la facette qui a la bonne couleur, et la face ext est noire
                [x,y,z,J,C] = facet(Pinf(:,1),Psup(:,1),Pinf(:,2),Jinf(1),Jsup(1),Jinf(2),Cinf(1),Csup(1),Cinf(2));
                Pinf(:,1) = []; Jinf(1) = []; Cinf(1) = [];
            else
                [x,y,z,J,C] = facet(Pinf(:,1),Psup(:,1),Psup(:,2),Jinf(1),Jsup(1),Jsup(2),Cinf(1),Csup(1),Csup(2));
                Psup(:,1) = []; Jsup(1) = []; Csup(1) = [];
            end
            X = [X x]; Y = [Y y]; Z = [Z z]; 
            JJ = [JJ J]; CC = [CC C];
        end        
    end
    // build facets around north pole
    Psup = Betas($);
    Pinf = [Betas($-1) Betas($-1)(:,1)];
    Jsup = Jmean($);
    Jinf = [Jmean($-1) Jmean($-1)(1)];
    Csup = constraints($);
    Cinf = [constraints($-1) constraints($-1)(1)];
    for j=1:size(Betas($-1),2)
        [x,y,z,J,C] = facet(Psup,Pinf(:,j+1),Pinf(:,j),Jsup,Jinf(j+1),Jinf(j),Csup,Cinf(j+1),Cinf(j));
        X = [X x]; Y = [Y y]; Z = [Z z]; JJ = [JJ J]; CC = [CC C];
    end
        
endfunction

function [x,y,z,J1,J2] = facet(A,B,C,J1a,J1b,J1c,J2a,J2b,J2c)
    x = [A(1);B(1);C(1)];
    y = [A(2);B(2);C(2)];
    z = [A(3);B(3);C(3)];
    J1 = [J1a; J1b; J1c];
    J2 = [J2a; J2b; J2c];
endfunction

function [fig,minJ,maxJ] = SIcircle(SImapFile,sources,map)
    // For 2 sources, plot :
    // - a unit circle = place of of the possible extraction coefficients
    // - a parametric curve : radius = NIAC(theta)
    // - the points corresponding to the extraction coefficients ,
    //    with their pNIAC and indicator max/min/unknown
    // INPUTS :
    // - SImapFile = data file
    // - sources = list of sources (strings)
    // - map = map parameters
    load(SImapFile);    // -> 'extraction_points','estMax','pSI','t','Betas','Jmean','constraints'
    // circle
    fig = scf();
    fig.axes_size = [600,600];
    angle = -%pi:%pi/100:%pi;
    plot(cos(angle),sin(angle),'k--');
    // parametric curve : radius = NIAC(theta)
    Rho = 1 + (Jmean-min(Jmean))/(max(Jmean)-min(Jmean));
    polarplot(t,Rho); 
    a = gca();
    a.children($-1).children.thickness = 3;
    // extraction points
    for k=1:2
        theta = atan(extraction_points(2,k),extraction_points(1,k));
        rho = 1 + (pSI(k)-min(Jmean))/(max(Jmean)-min(Jmean));
        x = rho*cos(theta); y = rho*sin(theta);      
        select estMax(k)
        case 1
            mark = map.targetBetaColor+'^';
        case -1
            mark = map.targetBetaColor+'v';
        case 0
            mark = map.targetBetaColor+'o';
        end  
        plot([-x x],[-y y],mark); 
        point = gce(); point.children.mark_size = 10;
        plot([-x x],[-y y],map.targetBetaColor+'--'); 
        xstring(x,y+0.1,string(pSI(k)));
        xstring(x,y,sources(k));
    end   
    minJ = min(Jmean);
    maxJ = max(Jmean);
    xlabel("$\beta_1$"); a.x_label.font_size = 4;
    ylabel("$\beta_2$"); a.y_label.font_size = 4;
    xtitle("$\textrm{Polar plot pNIAC}(\arctan(\beta_2/\beta_1))$"); 
    a.title.font_size = 4;
endfunction


function fig = simplified_SIsphere(SImapFile,sources)
    // For 3 sources, plot :
    // - a unit sphere = place of of the possible extraction coefficients
    // - the points corresponding to the extraction coefficients with beta_3 >= 0,
    //    with their pNIAC and indicator max/min/unknown
    // INPUTS :
    // - SImapFile = data file
    // - sources = list of sources (strings)
    load(SImapFile);    // -> 'extraction_points','estMax','pSI');
    fig=scf();
    fig.color_map = jetcolormap(256);
    deff("[x,y,z]=sph(phi,tet)",["x=cos(phi).*cos(tet)";..
         "y=cos(phi).*sin(tet)";..
         "z=sin(phi)"]);
    [xx,yy,zz]=eval3dp(sph,linspace(-%pi/2,%pi/2,30),linspace(0,%pi*2,60));
    //cc=(xx+zz+2)*32;
    cc=(xx+zz+2)*48;
    clf();plot3d1([xx xx],[yy yy],list([zz,zz],[cc cc]),theta=90,alpha=0,flag=[5,6,3])
    p = list();
    for k=1:3
        x = extraction_points(1,k);
        y = extraction_points(2,k);
        z = extraction_points(3,k);
        plot3d([x -x]*1.02,[y -y]*1.02,[z -z]*1.02,flag=[0,0]);
        p($+1) = gce();
        p($).surface_mode = "off";
        p($).mark_mode = "on";
        select estMax(k)
        case 1
            p($).mark_style = 6;
        case -1
            p($).mark_style = 7;
        case 0
           p($).mark_style = 9;
        end
        p($).thickness = 2;
        p($).mark_size = 2;
        p($).mark_foreground = 165+30*k;
        p($).mark_background = 165+30*k;                   
    end   
    // legend
    legends([sources(1),sources(2),sources(3)], [-p(1).mark_style, -p(2).mark_style, -p(3).mark_style], "ur") 
    e = gce();
    for k=1:3
        e.children(2*k).mark_foreground = 0;
        e.children(2*k).mark_background = p(k).mark_background;
        e.children(2*k).mark_size = 6;
        e.children(2*k-1).font_size = 3
    end
    // layout
    a = gca();
    a.margins =  [0.1,0.1,0.1,0.1];
    xtitle("NIAC simplified map");
    a.title.font_size = 4;
    xlabel("$\beta_1$");a.x_label.font_size = 4;
    ylabel("$\beta_2$");a.y_label.font_size = 4;
    zlabel("$\beta_3$");a.z_label.font_size = 4;    
endfunction


function fig = simplified_SIcircle(SImapFile,sources,map)
    // For 2 sources, plot :
    // - a unit circle = place of of the possible extraction coefficients
    // - the points corresponding to the extraction coefficients with beta_2 >= 0,
    //    with their pNIAC and indicator max/min/unknown
    // INPUTS :
    // - SImapFile = data file
    // - sources = list of sources (strings)
    // - map = map parameters
    load(SImapFile);    // -> 'extraction_points','estMax','pSI');
    // circle
    fig = scf();
    a = gca();
    fig.axes_size = [600,600];
    angle = -%pi:%pi/100:%pi;
    plot(cos(angle),sin(angle),'k');
    // extraction points
    for k=1:2
        x = extraction_points(1,k);
        y = extraction_points(2,k);
        select estMax(k)
        case 1
            mark = map.targetBetaColor+'^';
        case -1
            mark = map.targetBetaColor+'v';
        case 0
            mark = map.targetBetaColor+'o';
        end
        plot([-x x],[-y y],mark); 
        point = gce(); point.children.mark_size = 10;
        xstring(x,y+0.05,string(pSI(k)));
        xstring(x,y,sources(k));
    end   
    xlabel("$\beta_1$"); a.x_label.font_size = 4;
    ylabel("$\beta_2$"); a.y_label.font_size = 4;
endfunction

function  simplified_SIdisplay(SImapFile)
    // For p sources, display :
    // - the points corresponding to the extraction coefficients with beta_2 >= 0,
    //    with their pNIAC and indicator max/min/unknown
    load(SImapFile);    // -> 'extraction_points','estMax','pSI');
    for k=1:size(extraction_points,2)
        point = sci2exp(extraction_points(:,k)');
        select estMax(k)
        case 1
            label = "is max";
        case -1
            label = "is min";
        case 0
            label = "is ???";
        end
        disp(point + " " + label + ". pNIAC = " + string(pSI(k)));
    end   
endfunction



//**************** Maps (sphere projections and zooming areas display) **************

function SImap(points,BFGSpoints,BetaMax1,targetBeta,f)

    // INPUTS
    // each column of points = 1 point Beta of the optimisation process
    //                       = [Beta; J; L]
    // mainPoints = indices of the points kept at the end of each BFGS iteration
    // BetaMax1 : beta estimated from the first optimization
    // targetBeta = the true beta
    // f =  figure handle of the NIAC sphere

    // function :
    // For 3 sources, rotate the SIsphere so that BetaMax1 is a the top of the projected hemisphere
    // then project the upper hemisphere, the extraction points
    // and the last swarm (for PSO) or all points (for GD) of the 1st optimization process 
    // Finally display the zooming area

    alpha = acos(BetaMax1(3)) * 180/%pi;
    theta = atan(BetaMax1(2)/BetaMax1(1)) * 180/%pi;
    if BetaMax1(1)<0 then
        theta = theta + 180;
    end
    f.children(2).rotation_angles = [alpha, theta];

    [m,n] = size(points); 
    p = length(targetBeta); // number of sources
    // indicate the zoom area on the original figure
    Phi = asin([points(3,BFGSpoints),targetBeta(3)]);
    Theta = atan([points(2,BFGSpoints),targetBeta(2)],[points(1,BFGSpoints),targetBeta(1)]);
    phi_min = min(Phi); phi_max = max(Phi); Delta_phi = phi_max-phi_min;
    phi_min = phi_min - 0.2*Delta_phi; phi_max = phi_max + 0.2*Delta_phi;
    dphi = (phi_max-phi_min)/100; Vphi = phi_min:dphi:phi_max;
    theta_min = min(Theta); theta_max = max(Theta); Delta_theta = theta_max-theta_min;
    theta_min = theta_min - 0.2*Delta_theta; theta_max = theta_max + 0.2*Delta_theta;
    dtheta = (theta_max-theta_min)/100; Vtheta = theta_min:dtheta:theta_max;
    scf(f.figure_id);
    a = gca(); rot_angles = a.rotation_angles;
    param3d(cos(Vphi)*cos(theta_min),cos(Vphi)*sin(theta_min),sin(Vphi),0,0,"X@Y@Z",[0,0]);
    e=gce(); e.foreground=color('red');
    param3d(cos(Vphi)*cos(theta_max),cos(Vphi)*sin(theta_max),sin(Vphi),0,0,"X@Y@Z",[0,0]);
    e=gce(); e.foreground=color('red');
    param3d(cos(phi_min)*cos(Vtheta),cos(phi_min)*sin(Vtheta),sin(phi_min)*ones(1,101),0,0,"X@Y@Z",[0,0]);
    e=gce(); e.foreground=color('red');
    param3d(cos(phi_max)*cos(Vtheta),cos(phi_max)*sin(Vtheta),sin(phi_max)*ones(1,101),0,0,"X@Y@Z",[0,0]);
    e=gce(); e.foreground=color('red');
    a.rotation_angles = rot_angles;
    xlabel("$\beta_1$");
    ylabel("$\beta_2$");
    zlabel("$\beta_3$");
    a.x_label.font_size = 4;
    a.y_label.font_size = 4;
    a.z_label.font_size = 4;

endfunction


function [fig,last_points,R,zoomRect] = SImap_simple(SImapFile,points1,BetaMax1,points2,targetBeta,map,sources)
    
    // inputs :
    // SImapFile : data file for building simplified NIACmap
    // points1: points collection of the first optimization
    // BetaMax1 : beta estimated from the first optimization
    // points2: points collection of the 2nd optimization
    // targetBeta : the true beta
    // margin : relative extra-margin around the rectangle encompassing the points of interest, for zooming

    // function :
    // For 3 sources, rotate the NIACsphere so that BetaMax1 is a the top of the projected hemisphere
    // then project the upper hemisphere, the extraction points
    // and the last swarm (for PSO) or all points (for GD) of the 1st optimization process 
    // Finally display the zooming area
    
    // outputs :
    // fig : resulting figure handle
    // last points = handle of the last points of the 1st optimization process
    // R : rotation matrix to have BetaMax1 a the top of the projected hemisphere
    // zoomRect = zooming rectangular area = [xmin, ymin, xmax, ymax]
    
    exec("matRotation.sci");
    load(SImapFile);    // -> 'extraction_points','estMax','pSI');
    // Rotation matrix to have BetaMax1 a the top of the projected hemisphere
    R = matRotation(BetaMax1,[0;0;1]);
    // circle bounding the circular map
    fig = scf();
    fig.axes_size = [600,600];
    xtitle("Projection of the sphere around the optimum found by the 1st optimization")
    angle = -%pi:%pi/100:%pi;
    plot(cos(angle),sin(angle),'k');
    // extraction points
    for k=1:3
        select estMax(k)
        case 1
            mark = map.targetBetaColor+'^';
        case -1
            mark = map.targetBetaColor+'v';
        case 0
            mark = map.targetBetaColor+'o';
        end
        pt = R*extraction_points(:,k);
        plot(pt(1),pt(2),mark);
        point = gce(); point.children.mark_size = 10;
        xstring(pt(1),pt(2),[sources(k);string(pSI(k))]);            
    end    
    // last swarm (for PSO) or all points of 1st optimization process (for GD)
    pts = R * points1(1:3,:,$);
    plot(pts(1,:),pts(2,:),'bx');
    a = gca(); 
    last_points = a.children(1).children;
    p = length(targetBeta); // number of sources
    select p
    case 2
        pointsRot = points2(1:2,:);
        targetBetaRot = targetBeta;
    case 3
        pointsRot = R * points2(1:3,:);
        targetBetaRot = R*targetBeta;
    end
    // indicate the zoom area on the original figure
    xmin = min([pointsRot(1,:) targetBetaRot(1)]); xmax = max([pointsRot(1,:) targetBetaRot(1)]);
    ymin = min([pointsRot(2,:) targetBetaRot(2)]); ymax = max([pointsRot(2,:) targetBetaRot(2)]);
    w0 = (xmax-xmin); h0 = (ymax-ymin);
    xrect(xmin-map.margin*w0,ymax+map.margin*h0,w0*(1+map.margin),h0*(1+map.margin));
    e=gce(); e.foreground=color('green');
    xstring(xmin,ymax,"zoom area");
    zoomRect = [xmin-map.margin*w0,ymin-map.margin*h0,xmax+map.margin*w0,ymax+map.margin*h0]; 
endfunction


function disp_zoomArea(SImapFile,fig,map)
    // For 2 sources and full display, 
    // display the zooming angular sector
    // INPUTS :
    // - SImapFile = data file
    // - fig = current figure
    // - map = map parameters    
    load(SImapFile); // -> 'thetas','Jmean','resolution'
    scf(fig.figure_id);
    plot(cos(thetas),sin(thetas));
    e = gce(); e.children.thickness = 6;
    e.children.foreground=color(map.zoomBoundsColor);
    xstring(cos(thetas($/2)),sin(thetas($/2))+0.1,"zooming angular sector")
    e = gce(); e.font_foreground=color(map.zoomBoundsColor);
endfunction

function zoomRect = disp_zoomArea_simple(points2,targetBeta,fig,map)
    // inputs :
    // - points2: points collection of the 2nd optimization
    // - targetBeta : the true beta
    // - fig : current figure
    // - map : map parameters

    // function :
    // For 2 sources and simplified display, 
    // indicate the zoom area 
    
    // output :
    // zoomRect = zooming rectangular area = [xmin, ymin, xmax, ymax]
    scf(fig.figure_id);
    xmin = min([points2(1,:) targetBeta(1)]); xmax = max([points2(1,:) targetBeta(1)]);
    ymin = min([points2(2,:) targetBeta(2)]); ymax = max([points2(2,:) targetBeta(2)]);
    w0 = (xmax-xmin); h0 = (ymax-ymin);
    xrect(xmin-map.margin*w0,ymax+map.margin*h0,w0*(1+map.margin),h0*(1+map.margin));
    e=gce(); e.foreground=color(map.zoomBoundsColor);
    xstring(xmin,ymax,"zoom area");
    zoomRect = [xmin-map.margin*w0,ymin-map.margin*h0,xmax+map.margin*w0,ymax+map.margin*h0];    
endfunction


//*********** zoomed maps ********************

function f = zoomed_SImap(SImapFile,targetBeta,map)
    // For 3 sources and full display,
    // display the NIAC map zoomed around the optimum
    // INPUTS :
    // - SImapFile : data file
    // - targetBeta : the true beta
    // - map : map parameters
    exec("afficheMat.sci");
    load(SImapFile);    // -> 'R','Beta1','Beta2','Jmean','resolution'
    f = afficheMat((Jmean(:,:))',Beta2(1),Beta2($),Beta1(1),Beta1($));
    targetBetaRot = R*targetBeta;
    plot(targetBetaRot(1),targetBetaRot(2),'*')
    e = gce(); p = e.children;
    p.mark_style = map.targetBetaStyle;
    p.mark_foreground = 0;
    p.mark_background = color(map.targetBetaColor);
    p.mark_size = 10;
    a = gca(); a.font_size = 3;
    a.margins =  [0.1,0.1,0.1,0.15];
    xtitle("QN-BFGS (1 pixel = "+string(resolution)+")"); a.title.font_size = 4;
endfunction


function zoomed_SImap_simple(f,rect,last_points)

    // inputs :
    // f =  figure to zoom in
    // rect = rectangular zooming area
    // last_points = handle of last points, to unvisiblize

    // For 3 sources and simple display,
    // display the NIAC map zoomed around the optimum
    
    scf(f.figure_id);
    zoom_rect(rect);
    xtitle("clearing previous optimization process points")
    sleep(2000);
    // make previous optimization process points invisible
    last_points.visible = "off";
    xtitle("");
    xtitle("QN-BFGS"); a.title.font_size = 4;
    
endfunction    


function fig = zoomed_SIcurve(SImapFile,targetBeta)
    // For 2 sources and full display,
    // display the NIAC curve (function of arctan(beta2/beta1)) zoomed around the optimum
    // INPUTS :
    // - SImapFile : data file
    // - targetBeta : the true beta
    load(SImapFile); // -> 'thetas','Jmean','resolution'
    fig = scf();
    plot(thetas,Jmean,'k',"linewidth",2);
    // extraction point :
    // * its angle
    t = atan(targetBeta(2),targetBeta(1));
    // * the rho ~ Jmean of the closest angle from theta
    [m,k] = min(abs(exp(%i*thetas)-exp(%i*t)));
    rho = Jmean(k);
    // * vertical segment 
    plot([t t],[min(Jmean)+0.1*(rho-min(Jmean)) rho],'k--');
    xstring(t,min(Jmean),"extraction point")
    text = gce(); text.font_size = 4; text.alignment = "center";
    // layout
    a = gca();
    xtitle("QN-BFGS (zoom with angular resolution pi/"+string(%pi/resolution)+")"); 
    a.title.font_size = 4;
    a.font_size = 3;
    xlabel("$\textrm{atan}(\beta_2,\beta_1) \quad \textrm{(rad)}$"); a.x_label.font_size = 4;
    ylabel("pseudo-NIAC"); a.y_label.font_size = 4;
endfunction


//*************** Display optimization points ******************************


function last_points = display_1stOpt_GD(points,p,f,TimeUnit,map)
    // each column of points = 1 point Beta of the optimisation process
    // = [Beta; J; ...]
    // p = number of sources
    // f =  figure handle to display in
    // TimeUnit = time delay btw 2 successive points
    [m,n] = size(points); 
    realtimeinit(TimeUnit);
    realtime(0);
    scf(f.figure_id); a=gca();
    for i=1:n        
        select p
        case 2
            if map.type=="full" then
                theta = atan(points(2,i),points(1,i));
                rho = 1 + (points(3,i)-map.minJ)/(map.maxJ-map.minJ); 
                plot(rho*cos(theta),rho*sin(theta),map.opt1PointsColor+'x');
            else                
                plot(points(1,i),points(2,i),map.opt1PointsColor+'x');
            end
            e  = gce(); point = e.children;
            point.mark_size = 6;
        case 3 
            rot_angles = a.rotation_angles;               
            plot3d([points(1,i)*1.01 0],[points(2,i)*1.01 0],[points(3,i)*1.01 0],flag=[0,0]);
            a.rotation_angles = rot_angles;
            point = gce();
            point.surface_mode = "off";
            point.mark_mode = "on";
            point.mark_foreground = map.opt1PointsColorCode;
            point.mark_background = map.opt1PointsColorCode;                   
            point.mark_style = 2;
            point.mark_size = 1;
        end            
        point.thickness = 2;        
        realtime(i);
    end
    last_points = a.children(1:n);

endfunction


function last_points = display_1stOpt_PSO(points,p,f,TimeUnit,map)
    // dim of points : betas etc x particles x iterations
    // each column of points = 1 point Beta of the optimisation process
    // = [Beta; J; ...]
    // p = number of sources
    // f =  figure handle to display in
    // TimeUnit = time delay btw 2 successive iterations
    [m,n,q] = size(points); 
    realtimeinit(TimeUnit);
    realtime(0);
    scf(f.figure_id); a=gca();
    for it = 1:q    // iterations
        xtitle("iteration "+string(it));
        for i=1:n       // particles
            select p
            case 2
                if map.type=="full" then
                    theta = atan(points(2,i,it),points(1,i,it));
                    rho = 1 + (points(3,i,it)-map.minJ)/(map.maxJ-map.minJ); 
                    plot(rho*cos(theta),rho*sin(theta),'mx');
                else                
                    plot(points(1,i,it),points(2,i,it),'mx');
                end
                point = gce();
                point.children.mark_size = 6;
                point.children.thickness = 2;
            case 3    
                rot_angles = a.rotation_angles;              
                plot3d([points(1,i,it)*1.01 0],[points(2,i,it)*1.01 0],[points(3,i,it)*1.01 0],flag=[0,0]);
                a.rotation_angles = rot_angles;
                point = gce();
                point.surface_mode = "off";
                point.mark_mode = "on";
                point.mark_foreground = map.opt1PointsColorCode;
                point.mark_background = map.opt1PointsColorCode;                     point.mark_style = 2;
                point.mark_size = 1;
                point.thickness = 2;
            end            
        end
        realtime(it);
        a = gca();
        a.children(1:n).visible = "off";
    end
    last_points = a.children(1:n);
    last_points.visible = "on";

endfunction


function display_2ndOpt(points,mainPoints,R,TimeUnit,map)

    // inputs :
    // each column of points = 1 point Beta of the optimisation process
    // = [Beta; J; L]
    // mainPoints = indices of the points kept at the end of each BFGS iteration
    // R = rotation matrix (for 3 src)
    // f =  figure handle to display in
    // last_points = handle of last points, to unvisiblize
    // TimeUnit = time delay btw 2 successive points
    // colour = points colour
    
    // display 2nd optimization process
    
    fig = gcf();
    fig.figure_name = "2nd optimization : QN-BFGS";
    a = gca();
    a.title.text = [a.title.text ; "x = intermediate points in the linear search"];
    a.title.font_size = 3;
    [p,q] = size(R);
    select p
    case 2
        pointsRot = points(1:2,:);
    case 3
        pointsRot = R * points(1:3,:);
    end
    a = gca();
    bounds = a.data_bounds; // save figure bounds
    realtimeinit(TimeUnit);
    realtime(0)
    [m,n] = size(points); 
    for i=1:n        
        if mainPoints(1)==i then
            mainPoints = mainPoints(2:$);
            mark = '.'; mark_size = 12; 
        else 
            mark = 'x'; mark_size = 6;
        end
        if p==2 & map.type=="full" then
            theta = atan(pointsRot(2,i),pointsRot(1,i));
            rho = points(3,i); 
            plot(theta,rho,map.opt2PointsColor+mark);
            xstring(theta,rho,string(i));
        else
            plot(pointsRot(1,i),pointsRot(2,i),map.opt2PointsColor+mark);
            xstring(pointsRot(1,i),pointsRot(2,i),string(i));
        end
        a.children(2).children.mark_size = mark_size;
        a.children(1).font_color = 0;
        a.children(1).font_size = 2;
        realtime(i+2);
    end
    a.data_bounds = bounds; // restore fig bounds

endfunction




